from textwrap import dedent

import pytest


@pytest.fixture
def module_yaml_output():
    return dedent("""
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: dummy
          stream: rolling
          profiles:
            rolling: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: dummy
          stream: rolling
          version: 1
          context: abcdef12
          summary: <auto-generated module summary>
          description: >-
            <auto-generated module description>
          license:
            module:
            - MIT
            content:
            - <FILL THIS IN>
          profiles:
            everything:
              rpms:
              - python-django-bash-completion
          api:
            rpms:
            - python-django-bash-completion
          components:
            rpms:
              python-django:
                rationale: Present in the repository
          artifacts:
            rpms:
            - python-django-bash-completion-0:3.0.10-3.fc33.noarch
        ...
    """).lstrip()
